/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btcore.h,v 1.1 2002/07/12 08:27:12 kds Exp $

   HCI access library

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/

#ifndef	_BTCORE_H
#define	_BTCORE_H

#include <fcntl.h>
#include <pthread.h>
#include <net/if.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <syslog.h>

__BEGIN_DECLS

#include <affix/hci_types.h>
#include <affix/bluetooth.h>

extern int argc;
extern char **argv;
extern int btdaemon;
extern char *affix_version;
extern unsigned long btlog_mask;
extern int btlog_options;

extern char	btdev[IFNAMSIZ];
#define LINK_BLUETOOTH	0
#define LINK_TCP	1
#define LINK_IRDA	2
extern int	linkmode;
extern int	sdpmode;

extern unsigned char	hci_errno;
extern char 		*hci_errlist[];
extern char 		*lmp_compid[];

struct affix_tupla {
	int	id;
	char	*msg;
	char	*match;
};


extern struct affix_tupla codServiceClassMnemonic[];
extern struct affix_tupla codMajorClassMnemonic[];
extern struct affix_tupla codMinorComputerMnemonic[];
extern struct affix_tupla codMinorPhoneMnemonic[];
extern struct affix_tupla codMinorAudioMnemonic[];

#define NUM_LMPCOMPID	37



#define BTDEBUG_MODULE_CONTROL	0x01
#define BTDEBUG_MODULE_SDP	0x02
#define BTDEBUG_MODULE_OBEX	0x04
#define BTDEBUG_MODULE_HFP	0x08

#ifndef BTDEBUG_MODULE 
#define BTDEBUG_MODULE 0
#endif

#ifdef CONFIG_AFFIX_DEBUG
#define _BTDEBUG(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, fmt "\n", ##args);\
				} while (0)

#define _BTDEBUGn(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, fmt, ##args);\
				} while (0)

#define BTDEBUGn(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, __FUNCTION__": " fmt, ##args);\
				} while (0)
#define BTDEBUG(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, __FUNCTION__": " fmt "\n", ##args);\
				} while (0)

#define BTDUMP(data, len)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_hexdump(LOG_DEBUG, __FUNCTION__, data, len);\
				} while (0)
#else
#define _BTDEBUG(args...)
#define BTDEBUG(args...)
#define BTDUMP(data, len)
#endif

#define BTERROR(fmt, args...)		btlog_printf(LOG_ERR, __FUNCTION__ ": " fmt "\n", ##args)
#define BTWARN(fmt, args...)		btlog_printf(LOG_WARNING, __FUNCTION__ ": " fmt "\n", ##args)
#define BTNOTICE(fmt, args...)		btlog_printf(LOG_NOTICE, __FUNCTION__ ": " fmt "\n", ##args)
#define BTINFO(fmt, args...)		btlog_printf(LOG_INFO, __FUNCTION__ ": " fmt "\n", ##args)


// General purpose functions

int str2bda(BD_ADDR *p, char *str);
void bda2str(char *str, BD_ADDR *p);
void btlog_printf(int pr, const char *fmt, ...);
void btlog_hexdump(int priority, const char *fname, const unsigned char *data, int len);
int daemonize(void);

static inline int btsys_socket(int domain, int type, int protocol)
{
	return socket(domain, type, protocol);
}

static inline int btsys_ioctl(int fd, int cmd, void *arg)
{
	return ioctl(fd, cmd, arg);
}

static inline int btsys_poll(struct pollfd *ufds, unsigned int nfds, int timeout)
{
	return poll(ufds, nfds, timeout);
}

static inline int btsys_close(int fd)
{
	return close(fd);
}

static inline int HCI_Poll(struct pollfd *ufds, unsigned int nfds, int timeout)
{
	return btsys_poll(ufds, nfds, timeout);
}

static inline int HCI_Close(int fd)
{
	return btsys_close(fd);
}

static inline int HCI_ExecuteCommand(int fd, struct HCI_Command_Param *param)
{
	return btsys_ioctl(fd, BTIOC_EXEC, param);
}

static inline int HCI_ListenEvent(int fd, __u64 mask)
{
	return btsys_ioctl(fd, BTIOC_LISTEN, &mask);
}

static inline int HCI_RecvEvent(int fd, struct HCI_Recv_Event *param)
{
	return btsys_ioctl(fd, BTIOC_RECV_EVENT, param);
}

static inline int HCI_RecvMsg(int fd, __u8 *param)
{
	return btsys_ioctl(fd, BTIOC_RECV_MSG, param);
}

static inline int HCI_ChangeConnectRole(int fd, __u8 role)
{
	return btsys_ioctl(fd, BTIOC_CHANGE_ROLE, &role);
}

static inline int HCI_ReadConnectRole(int fd, __u8 *role)
{
	return btsys_ioctl(fd, BTIOC_READ_ROLE, role);
}

static inline int HCI_SetMode(int fd, int mode)
{
	return btsys_ioctl(fd, BTIOC_CTL_MODE, &mode);
}

static inline int HCI_ChangePacketType(int fd, int pkt_type)
{
	return btsys_ioctl(fd, BTIOC_PKT_TYPE, &pkt_type);
}



/* general hci */

static inline char *HCI_ErrorMsg(int errn)
{
	return hci_errlist[errn];
}

static inline unsigned char HCI_LastError(void)
{
	unsigned char	ret = hci_errno;
	hci_errno = 0;
	return ret;
}


/* Common Commands */
int HCI_GetDevices(int *devs);
int HCI_GetDeviceInfo(int hd, struct HCI_DeviceInfo *info);
int HCI_AudioMode(int fd, int mode, __u16 setting);
int HCI_GetConnectionInfo(int fd, BD_ADDR *bda);

/* ----------------------------------------------------------- */

/* Control */
int HCI_AddPINCode(BD_ADDR *bda, int Length, __u8 *Code);
int HCI_RemovePINCode(BD_ADDR *bda);
int HCI_RemoveLinkKey(BD_ADDR *bda);

int HCI_SetDBMask(__u32 dbmask);
int HCI_GetDBMask(__u32 *dbmask);

int HCI_OpenUART(char *name, int type, int speed, int flags, int proto);
int HCI_CloseUART(char *name);

/* L2CAP */
static inline int l2cap_getmtu(int fd)
{
	int		err, mtu;
	socklen_t	len;

	err = getsockopt(fd, SOL_AFFIX, BTSO_MTU, &mtu, &len);
	if (err < 0)
		return err;

	return mtu;
}

static inline int l2cap_setmtu(int fd, int mtu)
{
	return setsockopt(fd, SOL_AFFIX, BTSO_MTU, &mtu, sizeof(mtu));
}

static inline int l2cap_flush(int fd)
{
	return ioctl(fd, SIOCL2CAP_FLUSH, 0);
}

int l2cap_sendping(int fd, char *data, int size, int type);
int l2cap_ping(int fd, char *data, int size);

static inline int l2cap_singleping(int fd, char *data, int size)
{
	return l2cap_sendping(fd, data, size, L2CAP_SINGLEPING);
}


/* RFCOMM stuff */
int RFCOMM_BindPort(int fd, int line);
int RFCOMM_DisconnectPort(int line);
int RFCOMM_CheckPorts(struct RFCOMM_PortInfo *pi, int size);
int RFCOMM_GetMTU(int fd);
int RFCOMM_SetType(int fd, int type);


__END_DECLS

#endif
